SystemOrganization addCategory: #JSON!

!Integer methodsFor: '*JSON-writing' stamp: 'Igor.Stasenko 12/27/2009 08:05'!
jsonWriteOn: aWriteStream
	^ self printOn: aWriteStream base: 10
! !

!Association methodsFor: '*JSON-writing' stamp: 'matthias.berth 11/19/2009 10:21'!
jsonWriteOn: aStream
	self key asString jsonWriteOn: aStream.
	aStream nextPut: $:; space.
	self value jsonWriteOn: aStream.! !

!False methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:43'!
jsonWriteOn: aStream
	aStream nextPutAll: 'false'! !

!Number methodsFor: '*JSON-writing' stamp: 'ul 11/4/2008 04:46'!
jsonWriteOn: aWriteStream 

	self printOn: aWriteStream base: 10! !

OrderedDictionary subclass: #OrderedJsonObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!OrderedJsonObject class methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 1/9/2010 16:51'!
fromAssociations: collectionOfAssociations

	| result |
	result := self new.
	
	collectionOfAssociations do: [:each |
		result at: each key put: each value ].
	^ result! !

!OrderedJsonObject methodsFor: 'accessing' stamp: 'rh 2/19/2010 21:34'!
at: aKey

	"make it work more like javascript objects"
	^ self at: aKey ifAbsent: [nil]! !

!OrderedJsonObject methodsFor: 'accessing' stamp: 'ul 6/13/2018 02:04'!
at: aString put: anObject

	aString isString ifFalse: [ self error: 'String expected' ].
	super at: aString put: anObject.
	^self! !

!OrderedJsonObject methodsFor: 'error handling' stamp: 'klub 3/18/2016 22:14'!
doesNotUnderstand: aMessage

	| key precedence |
	key := aMessage selector.
	(precedence := key precedence) = 1 ifTrue: [ ^self at: key ifAbsent: nil ].
	(precedence = 3 and: [ (key indexOf: $:) = key size ]) ifTrue: [
		^self
			at: key allButLast asSymbol
			put: aMessage arguments first ].
	^super doesNotUnderstand: aMessage! !

!OrderedJsonObject methodsFor: 'accessing' stamp: 'rh 2/27/2010 00:03'!
name
"override 'cause Object defines this"
	^self at: 'name'! !

!OrderedJsonObject methodsFor: 'accessing' stamp: 'rh 2/27/2010 00:04'!
value
"override 'cause Object defines this"
	^self at: 'value'! !

!Time methodsFor: '*JSON' stamp: 'klub 6/17/2015 15:50'!
jsonWriteOn: stream

	stream nextPut: $".
	self print24: true showSeconds: true on: stream.
	stream nextPut: $"! !

!WriteStream methodsFor: '*json-printing' stamp: 'cwp 10/25/2006 12:27'!
jsonPrint: anObject
	anObject jsonWriteOn: self! !

!Text methodsFor: '*json-printing' stamp: 'cwp 10/26/2006 22:25'!
jsonWriteOn: aStream
	self string jsonWriteOn: aStream! !

!UndefinedObject methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:45'!
jsonWriteOn: aWriteStream 
	aWriteStream nextPutAll: 'null'! !

Error subclass: #JsonSyntaxError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonSyntaxError commentStamp: 'tonyg 4/29/2016 11:06' prior: 0!
Class Json signals (possibly-indirect) instances of me when reading a JSON value from an input stream fails.!

JsonSyntaxError subclass: #JsonIncompleteError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonIncompleteError commentStamp: 'tonyg 4/29/2016 11:37' prior: 0!
I signal that reading a JSON value failed because more input is required, but that the input seen so far was not incorrect. Compare to JsonInvalidError.

Be warned that reading numbers directly out of a stream can be ambiguous!!

Consider reading from '1234'. Is the result intended to be 1234, or is there missing input, and the next character will be '5', making the result (possibly) 12345?!

JsonSyntaxError subclass: #JsonInvalidError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonInvalidError commentStamp: 'tonyg 4/29/2016 11:06' prior: 0!
I signal that reading a JSON value failed because the input didn't match the (extended)JSON grammar implemented by class Json.!

!Dictionary methodsFor: '*JSON-writing' stamp: 'Igor.Stasenko 12/27/2009 08:10'!
jsonWriteOn: aStream

	| needComma |
	needComma := false.
	aStream nextPut: ${.
	self associationsDo: [ :assoc |
		needComma
			ifTrue: [ aStream nextPut: $, ]
			ifFalse: [ needComma := true ].
		assoc key jsonWriteOn: aStream.
		aStream nextPut: $:.
		assoc value jsonWriteOn: aStream ].
	aStream nextPut: $}.! !

Dictionary subclass: #JsonObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonObject class methodsFor: 'as yet unclassified' stamp: 'Igor.Stasenko 1/9/2010 16:51'!
fromAssociations: collectionOfAssociations

	| result |
	result := self new.
	
	collectionOfAssociations do: [:each |
		result at: each key put: each value ].
	^ result! !

!JsonObject methodsFor: 'accessing' stamp: 'rh 2/19/2010 21:34'!
at: aKey

	"make it work more like javascript objects"
	^ self at: aKey ifAbsent: [nil]! !

!JsonObject methodsFor: 'accessing' stamp: 'ul 6/7/2018 17:16'!
at: aString put: anObject

	aString isString ifFalse: [ self error: 'String expected' ].
	super at: aString put: anObject.
	^self! !

!JsonObject methodsFor: 'error handling' stamp: 'klub 3/18/2016 22:14'!
doesNotUnderstand: aMessage

	| key precedence |
	key := aMessage selector.
	(precedence := key precedence) = 1 ifTrue: [ ^self at: key ifAbsent: nil ].
	(precedence = 3 and: [ (key indexOf: $:) = key size ]) ifTrue: [
		^self
			at: key allButLast asSymbol
			put: aMessage arguments first ].
	^super doesNotUnderstand: aMessage! !

!JsonObject methodsFor: 'accessing' stamp: 'rh 2/27/2010 00:03'!
name
"override 'cause Object defines this"
	^self at: 'name'! !

!JsonObject methodsFor: 'forwarding' stamp: 'ct 8/30/2020 23:26'!
respondsTo: aSymbol

	^ (super respondsTo: aSymbol)
		or: [self includesKey: aSymbol]! !

!JsonObject methodsFor: 'accessing' stamp: 'rh 2/27/2010 00:04'!
value
"override 'cause Object defines this"
	^self at: 'value'! !

!True methodsFor: '*JSON-writing' stamp: 'tonyg 8/17/2005 00:44'!
jsonWriteOn: aStream
	aStream nextPutAll: 'true'! !

NumberParser subclass: #JsonNumberParser
	instanceVariableNames: 'fraction leadingZeroesAllowed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonNumberParser methodsFor: 'accessing' stamp: 'ul 3/29/2016 06:42'!
allowPlusSign

	^false! !

!JsonNumberParser methodsFor: 'accessing' stamp: 'ul 3/29/2016 06:42'!
allowPlusSignInExponent

	^true! !

!JsonNumberParser methodsFor: 'error' stamp: 'ul 3/29/2016 06:46'!
error: aString

	JsonSyntaxError signal: aString! !

!JsonNumberParser methodsFor: 'accessing' stamp: 'ul 3/29/2016 06:43'!
exponentLetters

	^'eE'! !

!JsonNumberParser methodsFor: 'parsing-private' stamp: 'ul 5/30/2020 02:41'!
fiveRaisedTo: anInteger

	anInteger >= 0 ifFalse: [ ^1 / (self fiveRaisedTo: 0 - anInteger) ].
	anInteger >= 28 ifTrue: [ ^5 raisedToInteger: anInteger ].
	^#(1 5 25 125 625 3125 15625 78125 390625 1953125 9765625 48828125 244140625 1220703125 6103515625 30517578125 152587890625 762939453125 3814697265625 19073486328125 95367431640625 476837158203125 2384185791015625 11920928955078125 59604644775390625 298023223876953125 1490116119384765625 7450580596923828125) at: anInteger + 1! !

!JsonNumberParser methodsFor: 'initialize-release' stamp: 'ul 5/29/2020 19:52'!
initialize

	super initialize.
	base := 10.
	fraction := Fraction numerator: 0 denominator: 1.
	leadingZeroesAllowed := false! !

!JsonNumberParser methodsFor: 'parsing-private' stamp: 'ul 5/30/2020 04:01'!
makeFloatFromMantissa: mantissa exponent: exponent
	"Return a Float equal to mantissa * 10 ^ exponent."

	exponent = 0 ifTrue: [ ^mantissa asFloat ].
	exponent > 0 ifTrue: [
		exponent <= 22 "Can 5 raisedToInteger: exponent be represented exactly as a Float? ((1 << Float precision - 1) log: 5) floor => 22"
			ifFalse: [ 
				exponent > 324 ifTrue: [ "(Float emax + Float precision * (2 log: 10)) ceiling => 324"
					mantissa isZero ifTrue: [ ^0.0 ].
					mantissa positive ifTrue: [ ^Float infinity ].
					^Float negativeInfinity ].
				^(mantissa * (self fiveRaisedTo: exponent)) asFloat timesTwoPower: exponent ]
			ifTrue: [
				mantissa highBitOfMagnitude <= Float precision ifTrue: [ "Mantissa can also be represented as an exact Float. Float >> #* should be exact."
					^mantissa asFloat * (self fiveRaisedTo: exponent) asFloat timesTwoPower: exponent ] ].
		^(mantissa * (self fiveRaisedTo: exponent)) asFloat timesTwoPower: exponent ].
	"exponent < 0"
	exponent >= -22 "Can 5 raisedToInteger: 0 - exponent be represented exactly as a Float?"
		ifTrue: [
			mantissa highBitOfMagnitude <= Float precision ifTrue: [ "Mantissa can also be represented as an exact Float. Float >> #/ should be exact."
				^mantissa asFloat / (self fiveRaisedTo: 0 - exponent) asFloat timesTwoPower: exponent ] ]
		ifFalse: [
			exponent < -324 ifTrue: [ "-1 * (Float precision - Float emin * (2 log: 10)) ceiling => -324"
				mantissa positive ifTrue: [ ^0.0 ].
				^-0.0 ] ].
	"No luck. Let Fraction >> #asFloat do the heavy lifting."
	^(fraction setNumerator: mantissa denominator: ((self fiveRaisedTo: 0 - exponent) bitShift: 0 - exponent)) asFloat "Can't use #timesTwoPower: here. No #reduced here, because there's none in NumberParser either"! !

!JsonNumberParser methodsFor: 'parsing-large int' stamp: 'ul 5/29/2020 19:53'!
nextElementaryLargeInteger
	"Form an unsigned integer with incoming digits from sourceStream.
	Return this integer, or zero if no digits found.
	Stop reading if end of digits or if a LargeInteger is formed.
	Count the number of digits and the position of lastNonZero digit and store them in instVar."

	| digit value |
	value := 0.
	nDigits := 0.
	lastNonZero := 0.
	[
		value isLarge ifTrue: [ ^value ].
		((digit := (sourceStream next ifNil: [ ^value ]) asInteger - 48 "$0 asInteger") < 0 
			or: [ digit >= 10 ]) ifTrue: [
			sourceStream skip: -1.
			^value ].
		nDigits := nDigits + 1.
		digit = 0
			ifFalse: [
				(lastNonZero = 0 and: [ nDigits > 1 and: [ leadingZeroesAllowed not ] ]) ifTrue: [ self error: 'Leading zeroes are not allowed.' ].
				lastNonZero := nDigits.
				value := value * 10 + digit ]
			ifTrue: [ value := value * 10 ] ] repeat! !

!JsonNumberParser methodsFor: 'parsing-public' stamp: 'ul 5/30/2020 03:09'!
nextNumber: negative

	| numberOfTrailingZeroInIntegerPart oldLeadingZeroesAllowed numberOfTrailingZeroInFractionPart numberOfNonZeroFractionDigits mantissa value |
	integerPart := self nextUnsignedIntegerOrNil ifNil: [ ^self error: 'Missing integer part!!' ].
	numberOfTrailingZeroInIntegerPart := nDigits - lastNonZero.
	(sourceStream peekFor: $.) ifFalse: [ "No fraction part"
		negative ifTrue: [ integerPart := integerPart negated ].
		self readExponent ifFalse: [ ^integerPart ].
		exponent > 0 ifTrue: [ ^integerPart * (self fiveRaisedTo: exponent) bitShift: exponent ].
		^self makeFloatFromMantissa: integerPart exponent: exponent ].
	oldLeadingZeroesAllowed := leadingZeroesAllowed.
	leadingZeroesAllowed := true.
	fractionPart := self nextUnsignedIntegerOrNil.
	leadingZeroesAllowed := oldLeadingZeroesAllowed.
	fractionPart ifNil: [ self error: 'Missing fraction part!!' ].
	fractionPart isZero ifTrue: [
		self readExponent ifFalse: [
			negative ifTrue: [
				integerPart isZero ifTrue: [ ^Float negativeZero ].
				^integerPart negated ].
			^integerPart ].
		exponent >= 0 ifTrue: [ 
			negative ifTrue: [ integerPart := integerPart negated ].
			^integerPart * (self fiveRaisedTo: exponent) bitShift: exponent ].
		value := self makeFloatFromMantissa: integerPart exponent: exponent.
		negative ifTrue: [ ^value negated ].
		^value ].
	numberOfTrailingZeroInFractionPart := nDigits - lastNonZero.
	numberOfNonZeroFractionDigits := lastNonZero.
	self readExponent.
	exponent := exponent - numberOfNonZeroFractionDigits.
	mantissa := (integerPart * (self fiveRaisedTo: numberOfNonZeroFractionDigits) bitShift: numberOfNonZeroFractionDigits) + (fractionPart // (self fiveRaisedTo: numberOfTrailingZeroInFractionPart) bitShift: 0 - numberOfTrailingZeroInFractionPart).
	value := self makeFloatFromMantissa: mantissa exponent: exponent.
	negative ifTrue: [ ^value negated ].
	^value! !

!JsonNumberParser methodsFor: 'parsing-public' stamp: 'ul 5/29/2020 18:26'!
nextUnsignedIntegerOrNil
	"Form an unsigned integer with incoming digits from sourceStream.
	Answer this integer, or nil if no digit found.
	Count the number of digits and the position of lastNonZero digit and store them in instVar"
	
	| nPackets high nDigitsHigh lastNonZeroHigh low |
	"read no more digits than one elementary LargeInteger"
	high := self nextElementaryLargeInteger.
	nDigits = 0 ifTrue: [^nil].
	
	"Not enough digits to form a LargeInteger, stop iteration"
	high isLarge ifFalse: [^high].

	"We now have to engage arithmetic with LargeInteger
	Decompose the integer in a high and low packets of growing size:"
	nPackets := 1.
	nDigitsHigh := nDigits.
	lastNonZeroHigh := lastNonZero.
	[
	low := self nextLargeIntegerBase: 10 nPackets: nPackets .
	high := (high * (self fiveRaisedTo: nDigits) bitShift: nDigits) + low.
	lastNonZero = 0 ifFalse: [lastNonZeroHigh := lastNonZero + nDigitsHigh].
	nDigitsHigh := nDigitsHigh + nDigits.
	low isLarge]
		whileTrue: [nPackets := nPackets * 2].

	nDigits := nDigitsHigh.
	lastNonZero := lastNonZeroHigh.
	^high! !

!ScaledDecimal methodsFor: '*JSON-writing' stamp: 'ul 1/22/2018 00:31'!
jsonWriteOn: stream

	self printOn: stream showingDecimalPlaces: scale! !

!Collection methodsFor: '*json' stamp: 'Igor.Stasenko 12/27/2009 08:03'!
jsonWriteOn: aStream
	"By default, use array braces "
	aStream nextPut: $[.
	
	self do: [:each |
		each jsonWriteOn: aStream
		  ] separatedBy: [ aStream nextPut: $, ].

	aStream nextPut: $]! !

TestCase subclass: #JsonTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonTests commentStamp: '<historical>' prior: 0!
I provide a number of test cases for class Json.!

!JsonTests methodsFor: 'helpers' stamp: 'tonyg 4/29/2016 11:12'!
assertIncompleteJson: aString
	self should: [self readFrom: aString] raise: JsonIncompleteError! !

!JsonTests methodsFor: 'helpers' stamp: 'tonyg 4/29/2016 11:12'!
assertInvalidJson: aString
	self should: [self readFrom: aString] raise: JsonInvalidError! !

!JsonTests methodsFor: 'helpers' stamp: 'ul 9/6/2020 01:53'!
json: aString equals: aValue
	| readValue |
	readValue := self readFrom: aString.
	self assert: aValue equals: readValue! !

!JsonTests methodsFor: 'helpers' stamp: 'tonyg 11/29/2005 18:03'!
readFrom: aString
	^ (Json newWithConstructors: {JsonDummyTestObject.}) readFrom: aString readStream
! !

!JsonTests methodsFor: 'helpers' stamp: 'tonyg 8/17/2005 16:24'!
render: anObject equals: aString
	self assert: (Json render: anObject) = aString! !

!JsonTests methodsFor: 'helpers' stamp: 'tonyg 11/29/2005 17:51'!
simpleDummyObject
	^ JsonDummyTestObject new a: 1; b: 2; c: 3; yourself! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 8/16/2005 23:51'!
testArray
	self json: '[]' equals: #().
	self json: '[[]]' equals: #(#()).
	self json: '[[], []]' equals: #(#() #()).
	self json: '["hi", "there"]' equals: #('hi' 'there').
	self json: '[["a", "b", null]]' equals: #(('a' 'b' nil)).! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 8/16/2005 23:42'!
testAtomFalse
	self json: 'false' equals: false.
	self json: '  false' equals: false.
	self json: 'false  ' equals: false.
	self json: '  false  ' equals: false.
! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 8/16/2005 23:43'!
testAtomNull
	self json: 'null' equals: nil.
	self json: '  null' equals: nil.
	self json: 'null  ' equals: nil.
	self json: '  null  ' equals: nil.
! !

!JsonTests methodsFor: 'tests' stamp: 'ul 3/28/2020 04:53'!
testAtomNumber
	self json: '1' equals: 1.
	self json: '123' equals: 123.
	self should: [ self json: '0123' equals: 123 ] raise: JsonSyntaxError. "No leading zeroes."
	self json: '1.23e2' equals: 123.
	self json: '-1' equals: -1.
	self json: '-0' equals: 0.
	self json: '[-1.2]' equals: #(-1.2).! !

!JsonTests methodsFor: 'tests' stamp: 'ul 9/6/2020 01:53'!
testAtomString

	self json: '"hi"' equals: 'hi'.
	self json: '"\""' equals: '"'.
	self json: '"\\"' equals: '\'.
	self json: '""' equals: ''.
	self json: '"a\u0004b"' equals: ({$a. Character value: 4. $b} as: String).
	self json: '"a\nb"' equals: ({$a. Character lf. $b} as: String).
	self json: '"a\uD834\uDD1Eb"' equals: ({$a. Character value: 16r1D11E. $b} as: String).
	! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 8/16/2005 23:36'!
testAtomTrue
	self json: 'true' equals: true.
	self json: '  true' equals: true.
	self json: 'true  ' equals: true.
	self json: '  true  ' equals: true.
! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 11/29/2005 17:58'!
testCtor
	self json: '@JsonDummyTestObject {"a": 1, "b": 2, "c": 3}' equals: self simpleDummyObject.
	self json: (Json render: self simpleDummyObject) equals: self simpleDummyObject.! !

!JsonTests methodsFor: 'tests' stamp: 'Igor.Stasenko 12/27/2009 08:19'!
testDictionary
	self json: '{}' equals: (JsonObject new).
	self json: '{"a": "a"}' equals: (JsonObject new at: 'a' put: 'a'; yourself).
	self json: '{"a": [[]]}' equals: (JsonObject new at: 'a' put: #(#()); yourself).
	self json: '{"a":"b", "b":"a"}' equals: (JsonObject new add: 'a'->'b'; add: 'b'->'a';yourself).! !

!JsonTests methodsFor: 'tests' stamp: 'ul 7/16/2018 15:53'!
testDictionaryClass

	| parser |
	parser := Json new.
	self assert: (parser readFrom: '{}' readStream) class == JsonObject.
	parser dictionaryClass: OrderedJsonObject.
	self assert: (parser readFrom: '{}' readStream) class == OrderedJsonObject.
	self assert: (parser readFrom: '{"x":{"y":{}}}' readStream) x y class == OrderedJsonObject! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 4/29/2016 11:19'!
testIncomplete
	self assertIncompleteJson: ''.
	self assertIncompleteJson: '  '.
	self assertIncompleteJson: '  tr'.
	self assertIncompleteJson: 'tru'.
	self assertIncompleteJson: '['.
	self assertIncompleteJson: '[true'.
	self assertIncompleteJson: '[true,'.
	self assertIncompleteJson: '{'.
	self assertIncompleteJson: '{"hi"'.
	self assertIncompleteJson: '{"hi":'.
	self assertIncompleteJson: '{"hi":true'.
	self assertIncompleteJson: '{"hi":true,'.
	self assertIncompleteJson: '{"hi":true,"'.
	self assertIncompleteJson: '"hello'.
	self assertIncompleteJson: '"hello '.
	self assertIncompleteJson: '"hello\'.
	self assertIncompleteJson: '"\u26'.! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 4/29/2016 11:26'!
testInvalid
	self assertInvalidJson: 'x'.
	self assertInvalidJson: '  x'.
	self assertInvalidJson: '  trx'.
	self assertInvalidJson: 'trux'.
	self assertInvalidJson: '.'.
	self assertInvalidJson: ':'.
	self assertInvalidJson: ','.
	self assertInvalidJson: ']'.
	self assertInvalidJson: '}'.
	self assertInvalidJson: '[x'.
	self assertInvalidJson: '[true t'.
	self assertInvalidJson: '[true,]'.
	self assertInvalidJson: '{]'.
	self assertInvalidJson: '{,'.
	self assertInvalidJson: '{"hi",'.
	self assertInvalidJson: '{"hi":x'.
	self assertInvalidJson: '{"hi":,'.
	self assertInvalidJson: '{"hi":true "'.
	self assertInvalidJson: '{"hi":true,}'.
	self assertInvalidJson: '\u263A'.! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 4/29/2016 11:27'!
testInvalidUnicodeEscapes
	self assertInvalidJson: '"\u26"'. "Note that naively reading four chars runs off the end here, so we might see JsonIncompleteError, which would be wrong."
	self assertInvalidJson: '"\u2Z"'.
	self assertInvalidJson: '"\u2Z44"'.! !

!JsonTests methodsFor: 'tests' stamp: 'ul 7/16/2018 15:58'!
testJsonObjectAtPutReturnsTheObject

	{ JsonObject. OrderedJsonObject } do: [ :jsonObjectClass |
		| json |
		json := 	jsonObjectClass new.
		self assert: (json at: 'foo' put: 1) == json.
		self assert: (json foo: 1) == json ]! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 4/29/2016 11:13'!
testMissingCtor
	self assertInvalidJson: '@Missing[]'! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 4/29/2016 11:10'!
testMissingCtorNoMap
	self 
		should: [Json new readFrom: '@Missing[]' readStream]
		raise: JsonInvalidError! !

!JsonTests methodsFor: 'tests' stamp: 'ul 7/16/2018 15:58'!
testNonStringKeysRaiseError

	{ JsonObject. OrderedJsonObject } do: [ :jsonObjectClass |
		| json |
		json := 	jsonObjectClass new.
		self should: [ json at: 1 put: 1 ] raise: Error.
		self assert: json isEmpty.
		json at: '1' put: 1.
		self assert: 1 equals: (json at: '1') ]! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 8/17/2016 14:18'!
testStreaming
	| j |
	j := Json new stream: 'truefalsetrue[]{}1.234 5.678"A""B"nullnull' readStream.
	self assert: j readAny equals: true.
	self assert: j readAny equals: false.
	self assert: j readAny equals: true.
	self assert: j readAny equals: #().
	self assert: j readAny equals: JsonObject new.
	self assert: j readAny equals: 1.234.
	self assert: j readAny equals: 5.678.
	self assert: j readAny equals: 'A'.
	self assert: j readAny equals: 'B'.
	self assert: j readAny equals: nil.
	self assert: j readAny equals: nil.! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 4/29/2016 11:27'!
testStringWithUnicode

	| unicodeString |
	unicodeString := (Unicode value: 16r263A) asString.
	self json: '"\u263A"' equals:unicodeString.
	self json: '"\u263a"' equals:unicodeString.
	self json: '"', unicodeString, '"' equals: unicodeString.
	self render: unicodeString equals: '"', unicodeString, '"'.! !

!JsonTests methodsFor: 'tests' stamp: 'matthias.berth 11/19/2009 10:21'!
testWriteAssociation
	self render: 'key' -> 'value' equals: '"key": "value"'.
	self render: 'key' -> 2 equals: '"key": 2'.
	"keys should be strings"
	self render: 42 -> 2 equals: '"42": 2'.
	"try to do _something_ for more complex keys"
	self render: #(42 43 44) -> 2 equals:  '"#(42 43 44)": 2'.
	
	
	! !

!JsonTests methodsFor: 'tests' stamp: 'tonyg 8/21/2009 22:55'!
testWriteString
	self render: '"' equals: '"\""'.
	self render: '\' equals: '"\\"'.
	self render: 'hi' equals: '"hi"'.
	self render: ({$a. Character lf. $b} as: String) equals: '"a\nb"'.
	self render: ({$a. Character value: 4. $b} as: String) equals: '"a\u0004b"'.! !

!String methodsFor: '*JSON' stamp: 'klub 4/24/2019 10:58'!
asSerializedJson
	" Assume that the receiver is a valid serialized json string.
	Return an object that understands #jsonWriteOn: and will
	write this string on its argument. "

	^SerializedJson on: self! !

!String methodsFor: '*JSON-writing' stamp: 'klub 7/23/2013 21:06'!
jsonWriteOn: aStream

	| index start |
	aStream nextPut: $".
	start := 1.
	[ (index := self indexOfAnyOf: Json escapeSet startingAt: start) = 0 ] whileFalse: [
		aStream next: index - start putAll: self startingAt: start.
		aStream nextPutAll: (Json escapeForCharacter: (self at: index)).
		start := index + 1 ].
	start <= self size ifTrue: [
		aStream next: self size + 1 - start putAll: self startingAt: start ].
	aStream nextPut: $".
! !

!String methodsFor: '*JSON' stamp: 'klub 10/19/2018 11:05'!
parseAsJson
	"Convenience"

	^Json readFrom: self readStream! !

!String methodsFor: '*JSON' stamp: 'klub 10/19/2018 11:06'!
parseAsOrderedJson
	"Convenience"

	^Json new
		dictionaryClass: OrderedJsonObject;
		readFrom: self readStream! !

Object subclass: #Json
	instanceVariableNames: 'stream currentCharacter arrayBufferStream stringBufferStream numberParser ctorMap dictionaryClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

Json class
	instanceVariableNames: 'escapeArray escapeSet'!

!Json commentStamp: 'ul 3/29/2016 04:38' prior: 0!
I support reading and writing JSON (see http://json.org) formatted data - strings, numbers, boolean, nil, arrays and dictionaries.
The implementation is mainly based on RFC 7159 (https://www.ietf.org/rfc/rfc7159.txt). It has been extended with syntax for invoking a prearranged list of constructors on read objects.
Character encoding is not handled here. That's something you have to do before reading or after writing. Mixed processing is possible, but it's entirely your responsibility to keep things correct.

Instance Variables
	stream:				<PositionableStream>
	currentCharacter:	<Character>
	arrayBufferStream:	<WriteStream>
	stringBufferStream: <ReadWriteStream>
	numberParser: <ExtendedNumberParser>
	ctorMap: <Dictionary>

For parsing, stream has to be initialized by sending #readFrom: to my instance. The parser will pre-read one character, which will be stored in currentCharacter, so the stream is expected to support position manipulation, hence it should be a PositionableStream.
arrayBufferStream is WriteStream on an Array, which holds the values during array parsing (see #readArray). When there's a recursive call, this stream is used as a stack to separate the elements parsed in the previous invocation from the current one.
stringBufferStream is a ReadWriteStream on a String, which holds the currently parsed String (see #readString), the currently parsed number (see #readNumber) or the name of the currently parsed constructor (see #readConstructor). Recursion has no effect on this.
numberParser is a cached instance of ExtendedNumberParser. It's initialized on stringBufferStream, so it can quickly parse the number stored in it.
ctorMap is the Dictionary holding the constructor classes by name.

arrayBufferStream, stringBufferStream and numberParser are initialized lazily, and should not be accessed externally. Since these are shared objects, along with stream and currentCharacter, holding state, therefore a single instance of me shouldn't be used to parse multiple streams at the same time.

For writing, you can use the methods in the class-side rendering category. To convert individual objects to a string, you can use #asJsonString, to serialize it on a stream, you can use #jsonWriteOn:.
!

Json class
	instanceVariableNames: 'escapeArray escapeSet'!

!Json class methodsFor: 'accessing' stamp: 'ul 11/9/2010 04:07'!
escapeForCharacter: c
	
	| asciiValue |
	(asciiValue := c asciiValue) < 128
		ifTrue: [ ^escapeArray at: asciiValue + 1 ].
	^nil! !

!Json class methodsFor: 'accessing' stamp: 'klub 7/23/2013 21:05'!
escapeSet 

	^escapeSet! !

!Json class methodsFor: 'class initialization' stamp: 'klub 7/23/2013 21:00'!
initialize
	"Json initialize."
	
	| newEscapeArray newEscapeSet |
	newEscapeArray := Array new: 128.
	newEscapeSet := CharacterSet new.
	(0 to: 31), #(127) do: [ :each |
		newEscapeArray at: each + 1 put: '\u', (each printStringHex padded: #left to: 4 with: $0).
		newEscapeSet add: (Character value: each) ].
	{
		$" -> '\"'.
		$\ -> '\\'.
		$/ -> '\/'. "A hack, so the generated JSON will always be HTML+Javascript compatible, because the generated JSON will not contain the seqence </script>."
		Character backspace -> '\b'.
		Character lf -> '\n'.
		Character newPage -> '\f'.
		Character cr -> '\r'.
		Character tab -> '\t'.
	} do: [ :each |
		newEscapeArray at: each key asciiValue + 1 put: each value.
		newEscapeSet add: each key ].
	escapeArray := newEscapeArray.
	escapeSet := newEscapeSet! !

!Json class methodsFor: 'accessing' stamp: 'ul 9/6/2020 01:51'!
mimeType

	^'application/json'! !

!Json class methodsFor: 'instance creation' stamp: 'dkb 12/25/2009 02:51'!
newWithConstructors: aCollection
	| m |
	m := Dictionary new.
	aCollection do: [:each |
		(each isKindOf: Association)
			ifTrue: [m add: each]
			ifFalse: [m at: each name asString put: each]].
	^ self new ctorMap: m; yourself.! !

!Json class methodsFor: 'instance creation' stamp: 'tonyg 11/29/2005 17:45'!
readFrom: aStream
	^ self new readFrom: aStream.! !

!Json class methodsFor: 'rendering' stamp: 'ul 3/29/2016 03:54'!
render: anObject

	^String streamContents: [ :stream |
		anObject jsonWriteOn: stream ]! !

!Json class methodsFor: 'rendering' stamp: 'tonyg 11/30/2005 16:32'!
render: anObject withConstructor: aConstructorName on: aStream
	aStream nextPutAll: '@', aConstructorName.
	anObject jsonWriteOn: aStream.
! !

!Json class methodsFor: 'rendering' stamp: 'dkb 12/25/2009 02:52'!
renderInstanceVariables: aCollection of: anObject on: aStream
	| map |
	map := Dictionary new.
	aCollection do: [:ivarName | map at: ivarName put: (anObject instVarNamed: ivarName)].
	self render: map withConstructor: anObject class name asString on: aStream! !

!Json methodsFor: 'private' stamp: 'ul 3/24/2016 18:15'!
arrayBufferStream

	^arrayBufferStream ifNil: [ arrayBufferStream := (Array new: 10) writeStream ]! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 02:36'!
consume: aString

	| message |
	1 to: aString size do: [ :index |
		(currentCharacter := stream next) == (aString at: index) ifFalse: [
			message := 'Expected ', (aString at: index)  printString.
			currentCharacter
				ifNil: [ self incomplete: message ]
				ifNotNil: [ self invalid: message ] ] ].
	currentCharacter := stream next! !

!Json methodsFor: 'accessing' stamp: 'tonyg 11/29/2005 11:40'!
ctorMap
	^ ctorMap! !

!Json methodsFor: 'accessing' stamp: 'dkb 12/25/2009 02:52'!
ctorMap: m
	ctorMap := m! !

!Json methodsFor: 'accessing' stamp: 'ul 6/13/2018 02:03'!
dictionaryClass: aClass
	"This method allows you to override the default dictionary class to be created while parsing a stream. Instances of aClass are expected to understand #at:put:."

	dictionaryClass := aClass! !

!Json methodsFor: 'error handling' stamp: 'ul 12/13/2017 02:39'!
error: aString

	(currentCharacter isNil and: [ stream atEnd ])
		ifTrue: [ self incomplete: aString ]
		ifFalse: [ self invalid: aString ]! !

!Json methodsFor: 'error handling' stamp: 'ul 12/13/2017 02:32'!
incomplete: aString

	JsonIncompleteError signal: aString! !

!Json methodsFor: 'initialize-release' stamp: 'ul 7/16/2018 15:52'!
initialize
	
	dictionaryClass := JsonObject
	! !

!Json methodsFor: 'private' stamp: 'ul 3/25/2016 00:45'!
interpretStringEscape: aCharacter

	aCharacter == $b ifTrue: [ ^Character backspace ].
	aCharacter == $n ifTrue: [ ^Character lf ].
	aCharacter == $f ifTrue: [ ^Character newPage ].
	aCharacter == $r ifTrue: [ ^Character cr ].
	aCharacter == $t ifTrue: [ ^Character tab ].
	aCharacter == $u ifTrue: [ ^self unescapeUnicode ].
	aCharacter == $" ifTrue: [ ^aCharacter ].
	aCharacter == $\ ifTrue: [ ^aCharacter ].
	aCharacter == $/ ifTrue: [ ^aCharacter ].
	self error: 'Unexpected escaped character: ', aCharacter asString
! !

!Json methodsFor: 'error handling' stamp: 'ul 12/13/2017 02:33'!
invalid: aString

	JsonInvalidError signal: aString! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 02:36'!
readAny

	self skipWhitespace.
	currentCharacter == ${ ifTrue: [ ^self readDictionary ].
	currentCharacter == $[ ifTrue: [ ^self readArray ].
	currentCharacter == $" ifTrue: [ ^self readString ].
	currentCharacter == $t ifTrue: [ self consume: 'rue'. ^true ].
	currentCharacter == $f ifTrue: [ self consume: 'alse'. ^false ].
	currentCharacter == $n ifTrue: [ self consume: 'ull'. ^nil ].
	currentCharacter == $- ifTrue: [ ^self readNumber: true ].
	(#($0 $1 $2 $3 $4 $5 $6 $7 $8 $9) instVarsInclude: currentCharacter) ifTrue: [
		^self readNumber: false ].
	currentCharacter == $@ ifTrue: [ ^self readConstructor ].
	self invalid: 'Unknown Json input'! !

!Json methodsFor: 'private' stamp: 'ul 12/11/2017 06:07'!
readArray

	| initialPosition |
	currentCharacter := stream next.
	self skipWhitespace.
	currentCharacter == $] ifTrue: [
		currentCharacter := stream next.
		^#() ].
	initialPosition := self arrayBufferStream position.
	[
		arrayBufferStream nextPut: self readAny.
		self skipWhitespace.
		currentCharacter == $] ifTrue: [ 
			|  result |
			result := arrayBufferStream originalContents copyFrom: initialPosition + 1 to: arrayBufferStream position.
			arrayBufferStream position: initialPosition.
			currentCharacter := stream next.
			^result ].
		currentCharacter == $, ifFalse: [ self error: 'Unexpected character: ', currentCharacter asString ].
		currentCharacter := stream next ] repeat.
! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 02:40'!
readConstructor

	ctorMap ifNil: [ ^self error: 'No constructors were declared.' ].
	self resetStringBufferStream.
	[ (currentCharacter := stream next) == $. or: [ currentCharacter isLetter ] ] whileTrue: [
		stringBufferStream nextPut: currentCharacter ].
	(ctorMap at: stringBufferStream contents ifAbsent: nil) ifNotNil: [ :constructor |
		^constructor constructFromJson: self readAny ].	
	self error: 'Unknown constructor: ', stringBufferStream contents! !

!Json methodsFor: 'private' stamp: 'ul 6/13/2018 02:01'!
readDictionary

	| result key commaNeeded |
	result := dictionaryClass new.
	commaNeeded := false.
	currentCharacter := stream next.
	[
		self skipWhitespace.
		currentCharacter == $} ifTrue: [ 
			currentCharacter := stream next.
			^result ].
		commaNeeded
			ifFalse: [ commaNeeded := true ]
			ifTrue: [ 
				currentCharacter == $, ifFalse: [ self error: 'Missing comma' ].
				currentCharacter := stream next.
				self skipWhitespace ].
		currentCharacter == $" ifFalse: [  self error: 'Key in dictionary must be string' ].
		key := self readString.
		self skipWhitespace.
		currentCharacter == $: ifFalse: [ self error: 'Missing colon' ].
		currentCharacter := stream next.
		result at: key put: self readAny ] repeat! !

!Json methodsFor: 'parsing' stamp: 'ul 8/18/2019 00:34'!
readFrom: aStream

	| result |
	stream := aStream.
	numberParser ifNotNil: [ numberParser on: stream ].
	currentCharacter := stream next.
	result := self readAny.
	currentCharacter ifNotNil: [ stream skip: -1 ]. "Undo prereading."
	^result! !

!Json methodsFor: 'private' stamp: 'ul 3/29/2016 21:05'!
readNumber: negative

	| result |
	negative ifFalse: [ stream skip: -1 ].
	result := (numberParser ifNil: [
		numberParser := JsonNumberParser new
			on: stream;
			failBlock: [ self error: 'Invalid number.' ];
			yourself ])
		nextNumber: negative.
	currentCharacter := stream next.
	^result
	! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 02:32'!
readString

	self resetStringBufferStream.
	[
		currentCharacter := stream next ifNil: [self incomplete: 'Unexpected end of string' ].
		currentCharacter == $" ifTrue: [ 
			currentCharacter := stream next.
			^stringBufferStream contents ].
		currentCharacter == $\
			ifTrue: [ 
				stringBufferStream nextPut: (self interpretStringEscape: (stream next ifNil: [
					self incomplete: 'Unexpected end of string' ])) ]
			ifFalse: [ stringBufferStream nextPut: currentCharacter ] ] repeat! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 02:30'!
resetStringBufferStream

	stringBufferStream 
		ifNil: [ stringBufferStream := ReadWriteStream on: (String new: 64) ]
		ifNotNil: [ stringBufferStream resetToStart ]! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 02:40'!
skipWhitespace

	currentCharacter ifNil: [
		self incomplete: 'Input stream is empty' ].
	[ currentCharacter isSeparator ] whileTrue: [
		currentCharacter := stream next ifNil: [
			self incomplete: 'Input stream is empty' ]. ]! !

!Json methodsFor: 'accessing' stamp: 'tonyg 8/16/2005 20:20'!
stream
	"Answer the value of stream"

	^ stream! !

!Json methodsFor: 'private' stamp: 'ul 12/13/2017 01:46'!
stream: aStream
	"For testing purposes only"

	stream := aStream.
	numberParser ifNotNil: [ numberParser on: stream ].
	currentCharacter := stream next! !

!Json methodsFor: 'private' stamp: 'ul 9/6/2020 01:56'!
unescapeUnicode

	| code digitValue lowSurrogateCode |
	code := 0.
	1 to: 4 do: [ :index |
		(digitValue := (stream next ifNil: [ self incomplete: 'Unexpected end of stream' ]) digitValue) < 0 ifTrue: [
			self invalid: 'Invalid hexadecimal digit' ].
		digitValue < 16
			ifTrue: [ code := code * 16 + digitValue ]
			ifFalse: [ self invalid: 'Invalid hexadecimal digit' ] ].
	code < 16rD800 ifTrue: [ ^code asCharacter ].
	code > 16rDFFF ifTrue: [ ^code asCharacter ].
	code <= 16rDBFF ifFalse: [ self invalid: 'High surrogate value expected' ].
	"Parse the escaped low surrogate"
	(stream next ifNil: [ self incomplete: 'Unexpected end of string' ]) == $\ 
		ifFalse: [ self invalid: 'Escaped low surrogate expected.' ].
	(stream next ifNil: [ self incomplete: 'Unexpected end of string' ]) == $u
		ifFalse: [ self invalid: 'Escaped low surrogate expected.' ].
	lowSurrogateCode := 0.
	1 to: 4 do: [ :index |
		(digitValue := (stream next ifNil: [ self incomplete: 'Unexpected end of stream' ]) digitValue) < 0 ifTrue: [
			self invalid: 'Invalid hexadecimal digit' ].
		digitValue < 16
			ifTrue: [ lowSurrogateCode := lowSurrogateCode * 16 + digitValue ]
			ifFalse: [ self invalid: 'Invalid hexadecimal digit' ] ].
	(lowSurrogateCode >= 16rDC00 and: [ lowSurrogateCode <= 16rDFFF ]) 
		ifFalse: [ ^self invalid: 'Escaped low surrogate expected.' ].
	^((code - 16rD800 bitShift: 10) + lowSurrogateCode + '16r2400') asCharacter! !

Object subclass: #JsonDummyTestObject
	instanceVariableNames: 'a b c'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!JsonDummyTestObject class methodsFor: 'as yet unclassified' stamp: 'tonyg 11/29/2005 17:49'!
constructFromJson: j
	^ self new a: (j at: 'a'); b: (j at: 'b'); c: (j at: 'c'); yourself! !

!JsonDummyTestObject methodsFor: 'as yet unclassified' stamp: 'tonyg 11/29/2005 17:56'!
= other
	^ other class == self class and: [
		a = other a and: [
		b = other b and: [
		c = other c]]]! !

!JsonDummyTestObject methodsFor: 'accessing' stamp: 'tonyg 11/29/2005 17:48'!
a
	"Answer the value of a"

	^ a! !

!JsonDummyTestObject methodsFor: 'accessing' stamp: 'dkb 12/25/2009 02:56'!
a: anObject
	"Set the value of a"

	a := anObject! !

!JsonDummyTestObject methodsFor: 'accessing' stamp: 'tonyg 11/29/2005 17:48'!
b
	"Answer the value of b"

	^ b! !

!JsonDummyTestObject methodsFor: 'accessing' stamp: 'dkb 12/25/2009 02:56'!
b: anObject
	"Set the value of b"

	b := anObject! !

!JsonDummyTestObject methodsFor: 'accessing' stamp: 'tonyg 11/29/2005 17:48'!
c
	"Answer the value of c"

	^ c! !

!JsonDummyTestObject methodsFor: 'accessing' stamp: 'dkb 12/25/2009 02:56'!
c: anObject
	"Set the value of c"

	c := anObject! !

!JsonDummyTestObject methodsFor: 'as yet unclassified' stamp: 'tonyg 11/30/2005 16:42'!
jsonWriteOn: s
	Json renderInstanceVariables: {#a. #b. #c} of: self on: s
! !

!Object methodsFor: '*JSON' stamp: 'Igor.Stasenko 1/9/2010 16:18'!
asJsonString

	^ String streamContents: [:str |
		self jsonWriteOn: str ]! !

Object subclass: #SerializedJson
	instanceVariableNames: 'jsonString'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'JSON'!

!SerializedJson class methodsFor: 'instance creation' stamp: 'klub 11/4/2017 10:58'!
on: aString

	^self new
		jsonString: aString;
		yourself! !

!SerializedJson methodsFor: 'accessing' stamp: 'klub 11/4/2017 10:57'!
jsonString
	
	^jsonString
! !

!SerializedJson methodsFor: 'accessing' stamp: 'klub 11/4/2017 10:57'!
jsonString: aString
	
	jsonString := aString
! !

!SerializedJson methodsFor: 'json-writing' stamp: 'klub 11/4/2017 10:58'!
jsonWriteOn: stream

	stream nextPutAll: jsonString! !
Json initialize!
